// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef WM_WINDOW_TREE_H
#define WM_WINDOW_TREE_H

#include <QObject>

class WMWindowTree : QObject
{
    Q_OBJECT

public:
    WMWindowTree();
};

#endif // WM_WINDOW_TREE_H
